/*
 * This file is part of Technology Advancement GS, which is a GameScript 
 * for OpenTTD Copyright (C) 2012-2013  Leif Linse
 *
 * Technology Advancement GS is free software; you can redistribute it  
 * and/or modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; version 2 of the License
 *
 * Technology Advancement GS is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Technology Advancement GS; If not, see  
 * <http://www.gnu.org/licenses/> or write to the Free Software Foundation,  
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
 
 const MODEL_LIFE = 45;
 
 enum EngineType {
	RAIL_ENGINE = "Locomotive",
	RAIL_WAGON = "Wagon",
	ROAD_VEHICLE = "Road vehicle",
	SHIP = "Ship",
	PLANE = "Plane"
}
 
 /*
  * For each engine in the game, an instance of this class is created.
  */
class Engine
{
	id = null;
	engine_type = null;
	available = null;
	design_date = null;
	model_life = null;
	retire_date = null;
	
	constructor(engine_id, engine_type, technology_date, pause_after_year) {
		this.id = engine_id;
		this.design_date = GSEngine.GetDesignDate(this.id);
		this.available = GSEngine.IsBuildable(this.id);
		
		this.SetEngineType(engine_type);
		this.SetModelLife();
		this.SetRetireDate(pause_after_year);
		
		Log.Info("Engine id: " + this.id
			   + ", name:" + GSEngine.GetName(this.id)
			   + ", engine_type: " + this.engine_type
			   + ", available: " + this.available
			   + ", design_date: " + GSDate.GetYear(this.design_date) + "." + GSDate.GetMonth(this.design_date) + "." + GSDate.GetDayOfMonth(this.design_date)
			   + ", model_life: " + this.model_life
			   + ", retire_date: " + GSDate.GetYear(this.retire_date) + "." + GSDate.GetMonth(this.retire_date) + "." + GSDate.GetDayOfMonth(this.retire_date)
			   , Log.LVL_SUB_DECISIONS );
	}
} 

function Engine::SetEngineType(engine_type)
{
	switch (engine_type) {
		case GSVehicle.VT_RAIL:
			if (GSEngine.IsWagon) {
				this.engine_type = EngineType.RAIL_WAGON;
			} else {
				this.engine_type = EngineType.RAIL_ENGINE;
			}
			break;
		case GSVehicle.VT_ROAD:
			this.engine_type = EngineType.ROAD_VEHICLE;
			break;
		case GSVehicle.VT_WATER:
			this.engine_type = EngineType.SHIP;
			break;
		case GSVehicle.VT_AIR:
			this.engine_type = EngineType.PLANE;
			break;
	}
}

function Engine::SetModelLife()
{
	switch (this.engine_type) {
		case EngineType.RAIL_ENGINE:
			this.model_life = GSController.GetSetting("rail_engine_model_life");
			break;
		case EngineType.RAIL_WAGON:
			this.model_life = GSController.GetSetting("rail_wagon_model_life");
			break;
		case EngineType.ROAD_VEHICLE:
			this.model_life = GSController.GetSetting("road_vehicle_model_life");
			break;
		case EngineType.SHIP:
			this.model_life = GSController.GetSetting("ship_model_life");
			break;
		case EngineType.PLANE:
			this.model_life = GSController.GetSetting("plane_model_life");
			break;
	}
}

function Engine::SetRetireDate(pause_after_year)
{
	this.retire_date = GSDate.GetDate(GSDate.GetYear(this.design_date) + this.model_life, GSDate.GetMonth(this.design_date), GSDate.GetDayOfMonth(this.design_date));
	if ((pause_after_year != 0 && this.retire_date >= GSDate.GetDate(pause_after_year, 1, 1)) || this.model_life == 0) {
		this.retire_date = -1;
	}
}

function Engine::CheckEngineAvailable(date, never_expire)
{
	if (date > this.design_date && (never_expire || this.retire_date < 0 || date < this.retire_date)) {
		return true;
	}
	return false;
}

function Engine::CheckEngine(company_list, date, never_expire)
{
	local make_available = this.CheckEngineAvailable(date, never_expire);
	if (make_available != this.available) {
		if (make_available) {
			foreach (company_id in company_list) {
				GSEngine.EnableForCompany(this.id, company_id);
			}
			Log.Info(GSEngine.GetName(this.id) + " is now enabled", Log.LVL_SUB_DECISIONS );
		} else {
			foreach (company_id in company_list) {
				GSEngine.DisableForCompany(this.id, company_id);
			}
			Log.Info(GSEngine.GetName(this.id) + " is now disabled", Log.LVL_SUB_DECISIONS );
		}
		this.available = make_available;
	}
}

function Engine::SetupEnginesForCompany(company_id, date, never_expire, all_enabled)
{
	if (all_enabled || this.CheckEngineAvailable(date, never_expire)) {
		GSEngine.EnableForCompany(this.id, company_id);
	} else {
		GSEngine.DisableForCompany(this.id, company_id);
	}
}








