/*
 * This file is part of Technology Advancement GS, which is a GameScript 
 * for OpenTTD Copyright (C) 2012-2013  Leif Linse
 *
 * Technology Advancement GS is free software; you can redistribute it  
 * and/or modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; version 2 of the License
 *
 * Technology Advancement GS is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Technology Advancement GS; If not, see  
 * <http://www.gnu.org/licenses/> or write to the Free Software Foundation,  
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/** Import SuperLib for GameScript **/
import("util.superlib", "SuperLib", 40);
Log <- SuperLib.Log;

/** Import other libs **/


/** Import other source code files **/
require("version.nut"); // get SELF_VERSION
require("engine.nut");
require("techdate.nut");

class MainClass extends GSController 
{
	_init_done = null;
	_engines = null;
	_technology_date = null;
	_loaded_data = null;
	_loaded_from_version = null;
	_company_list = null;
	_vehicles_never_expire = null;
	_all_technology_enabled = null;
	_pause_after_year = null;
	_technology_start_year = null;

	constructor()
	{
		this._init_done = false;
		this._engines = [];
		this._technology_date = null;
		this._loaded_data = null;
		this._loaded_from_version = null;
		this._company_list = [];
		this._vehicles_never_expire = null;
		this._all_technology_enabled = null;
		this._pause_after_year = null;
		this._technology_start_year = null;
	}
}

function MainClass::Start()
{
	// Initializing the script
	local start_tick = GSController.GetTick();
	GSGame.Pause();
	
	this.Init();
	
	GSGame.Unpause();
	this._init_done = true;
	local setup_duration = GSController.GetTick() - start_tick;
	Log.Info("Game setup done.", Log.LVL_INFO);
	Log.Info("Setup took " + setup_duration + " ticks.", Log.LVL_DEBUG);
	Log.Info("Happy playing !", Log.LVL_INFO);

	// Wait for the game to start 
	GSController.Sleep(1);

	// Main Game Script loop
	local last_loop_date = GSDate.GetCurrentDate();
	while (true) {
		local loop_start_tick = GSController.GetTick();

		// Handle incoming messages from OpenTTD
		this.HandleEvents();
		
		local never_expire = GSGameSettings.GetValue("vehicle.never_expire_vehicles");
		if (!this._all_technology_enabled && (this._technology_date.AdvanceDate() || this._vehicles_never_expire != never_expire)) {
			this._vehicles_never_expire = never_expire;
			foreach (engine in this._engines) {
				engine.CheckEngine(this._company_list, this._technology_date.date, this._vehicles_never_expire);
			}
		}
	
		// Loop with a frequency of one day
		local ticks_used = GSController.GetTick() - loop_start_tick;
		Log.Info("Ticks used by main loop: " + ticks_used, Log.LVL_DEBUG);
		GSController.Sleep(max(1, 74 - ticks_used));
	}
}

function MainClass::Init()
{
	GSGameSettings.SetValue("difficulty.vehicle_breakdowns", 0);
	local vehicles_never_expire = GSGameSettings.GetValue("vehicle.never_expire_vehicles");
	
	this._all_technology_enabled = GSController.GetSetting("enable_all_technology");
	this._pause_after_year = GSController.GetSetting("pause_after_year");
	this._technology_start_year = GSController.GetSetting("technology_start_year");
	
	if (this._loaded_data != null) { // Loaded Game
		this._company_list = this._loaded_data.company_list;
		this._technology_date = TechDate(this._loaded_data.technology_date);
	} else { // New Game
		if (this._technology_start_year > 0) {
			this._technology_date = TechDate(GSDate.GetDate(this._technology_start_year, 1, 1));
		} else {
			this._technology_date = TechDate(GSDate.GetCurrentDate());
		}
	}
	
	for (local vehicle_type = 0 ; vehicle_type < 4 ; vehicle_type++) { // VT_RAIL, VT_ROAD, VT_WATER, VT_AIR
		local engine_list = GSEngineList(vehicle_type);
		foreach (e, _ in engine_list) {
			if (GSEngine.IsValidEngine(e))
			{
				this._engines.append(Engine(e, vehicle_type, this._technology_date.date, this._pause_after_year));
			}
		}
	}
}

function MainClass::HandleEvents()
{
	if(GSEventController.IsEventWaiting()) {
		local ev = GSEventController.GetNextEvent();
		if (ev == null) return;

		local ev_type = ev.GetEventType();
		switch (ev_type) {
			case GSEvent.ET_COMPANY_NEW: {
				local company_event = GSEventCompanyNew.Convert(ev);
				this._company_list.append(company_event.GetCompanyID());
				local start_tick = GSController.GetTick();
				foreach (engine in this._engines) {
					engine.SetupEnginesForCompany(company_event.GetCompanyID(), this._technology_date.date, this._vehicles_never_expire, this._all_technology_enabled);
				}
				local setup_duration = GSController.GetTick() - start_tick;
				Log.Info("New company setup took " + setup_duration + " ticks.", Log.LVL_DEBUG);
				break;
			}
			case GSEvent.ET_COMPANY_BANKRUPT:
			case GSEvent.ET_COMPANY_MERGER: {
				local company_event = GSEventCompanyNew.Convert(ev);
				local company_id = company_event.GetCompanyID();
				local idx = 0;
				foreach (id in this._company_list) {
					if (id == company_id) {
						this._company_list.remove(idx);
						break;
					}
					++idx;
				}
				break;
			}
		}
	}
}

function MainClass::Save()
{
	Log.Info("Saving data to savegame", Log.LVL_INFO);

	// In case (auto-)save happens before we have initialized all data,
	// save the raw _loaded_data if available or an empty table.
	if (!this._init_done) {
		return this._loaded_data != null ? this._loaded_data : {};
	}

	return { 
		technology_date = this._technology_date.date,
		company_list = this._company_list
	};
}

function MainClass::Load(version, tbl)
{
	Log.Info("Loading data from savegame made with version " + version + " of the game script", Log.LVL_INFO);

	// Store a copy of the table from the save game
	// but do not process the loaded data yet. Wait with that to Init
	// so that OpenTTD doesn't kick us for taking too long to load.
	this._loaded_data = {}
   	foreach(key, val in tbl) {
		this._loaded_data.rawset(key, val);
	}

	this._loaded_from_version = version;
}
